import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.flashr.core.FlashrRequest;
import com.kelvinluck.flashr.core.FlashrResponseEvent;
/**
* Class: FlashrCache
* Class to cache relevant queries to flickr.com.
* 
* To use just set <Flashr.cacheQueries> to true.
* 
* Doing this means if you repeat a call to a given method with the same parameters then rather than
* spawning a repeated HTTP request to flickr.com the Flashr library will return the same results
* as the last call.
* 
* This has the advantage of reducing network traffic with flickr which can be a bottleneck in certain
* types of applications. The downside is increased memory usage on the client as the results of all
* queries to flickr are held in memory. For each application choose which is best for your needs -
* <Flashr.cacheQueries> set to true or false
* 
* By default, calls to the following methods are cached:
* 
* flickr.contacts.getList
* flickr.contacts.getPublicList
* flickr.favorites.getList
* flickr.favorites.getPublicList
* flickr.groups.pools.getContext
* flickr.groups.pools.getGroups
* flickr.groups.pools.getPhotos
* flickr.interestingness.getList
* flickr.people.findByEmail
* flickr.people.findByUsername
* flickr.people.getInfo
* flickr.people.getPublicGroups
* flickr.people.getPublicPhotos
* flickr.people.getUploadStatus
* flickr.photos.getAllContexts
* flickr.photos.getContactsPhotos
* flickr.photos.getContactsPublicPhotos
* flickr.photos.getContext
* flickr.photos.getCounts
* flickr.photos.getExif
* flickr.photos.getInfo
* flickr.photos.getNotInSet
* flickr.photos.getPerms
* flickr.photos.getRecent
* flickr.photos.getSizes
* flickr.photos.getUntagged
* flickr.photos.search
* flickr.photosets.getContext
* flickr.photosets.getInfo
* flickr.photosets.getList
* flickr.photosets.getPhotos
* flickr.tags.getListPhoto
* flickr.tags.getListUser
* flickr.tags.getListUserPopular
* flickr.tags.getRelated
*
* If you would like more or less methods cached you can edit the <cachedMethods> variable.
* 
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.core.FlashrCache 
{
	private static var _instance:FlashrCache;
	private var _cache:Object;
	
	/**
	* Variable: cachedMethods
	* Holds a list of all the methods whose results are cached.
	**/
	var cachedMethods:Array = ["flickr.contacts.getList", "flickr.contacts.getPublicList", "flickr.favorites.getList", "flickr.favorites.getPublicList", "flickr.groups.pools.getContext", "flickr.groups.pools.getGroups", "flickr.groups.pools.getPhotos", "flickr.interestingness.getList", "flickr.people.findByEmail", "flickr.people.findByUsername", "flickr.people.getInfo", "flickr.people.getPublicGroups", "flickr.people.getPublicPhotos", "flickr.people.getUploadStatus", "flickr.photos.getAllContexts", "flickr.photos.getContactsPhotos", "flickr.photos.getContactsPublicPhotos", "flickr.photos.getContext", "flickr.photos.getCounts", "flickr.photos.getExif", "flickr.photos.getInfo", "flickr.photos.getNotInSet", "flickr.photos.getPerms", "flickr.photos.getRecent", "flickr.photos.getSizes", "flickr.photos.getUntagged", "flickr.photos.search", "flickr.photosets.getContext", "flickr.photosets.getInfo", "flickr.photosets.getList", "flickr.photosets.getPhotos", "flickr.tags.getListPhoto", "flickr.tags.getListUser", "flickr.tags.getListUserPopular", "flickr.tags.getRelated"];
	 
	/**
	* Function: FlashrCache
	* Private constructor - class is a Singleton so instead use <FlashrCache.getInstance>
	**/
	private function FlashrCache()
	{
		_cache = {};
		LogWrapper.getLog().debug(this+" created");
	}
	
	/**
	* Function: clearCache
	* Clears the cache
	**/
	function clearCache():Void
	{
		_cache = {};
	}
	
	/**
	* Function: getCached
	* Returns the cached <FlashrResponseEvent> for a given <FlashrRequest> or undefined if this request
	* hasn't been cached yet.
	* 
	* Parameters:
	* request				-	The <FlashrRequest> that you want to check.
	* 
	* Returns:
	* Either the <FlashrResponseEvent> that was cached for this request or undefined if this <FlashrRequest> 
	* hasn't been cached yet.
	**/
	function getCached(request:FlashrRequest):FlashrResponseEvent
	{
		return _cache[request.querystring];
	}
	/**
	* Function: cache
	* Caches the passed <FlashrRequest> and it's <FlashrResponseEvent> as long as it is a method that
	* should be cached.
	* 
	* Parameters:
	* responseEvent			-	The <FlashrResponseEvent> that holds all information about the request response.
	**/
	function cache(responseEvent:FlashrResponseEvent):Void
	{
		if (_methodIsCachable(responseEvent.request.method)) {
			LogWrapper.getLog().debug("FlashrCache.cache - caching call to "+responseEvent.request);
			_cache[responseEvent.request.querystring] = responseEvent;
		}
	}
	
	/**
	* Function: _methodIsCachable
	* Private, internal function which checks whether calls to a given method should be cached.
	* 
	* Parameters:
	* methodName			-	The name of the method.
	* 
	* Returns:
	* Whether or not calls to this method should be cached.
	**/
	private function _methodIsCachable(methodName:String):Boolean
	{
		for (var i:Number=0; i<cachedMethods.length; i++) {
			if (cachedMethods[i] == methodName) {
				return true;
			}
		}
		return false;
	}
	
	/**
	* Function: getInstance
	**/
	static function getInstance():FlashrCache
	{
		if (_instance == null) {
			_instance = new FlashrCache();
		}
		return _instance;
	}
	
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flashr.core.FlashrCache]";
	}
}