import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.flashr.core.Flashr;
/**
* Class: com.kelvinluck.flashr.core.FlashrRequest
* Class to hold information about requests to the Flickr API through Flashr
*
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.core.FlashrRequest 
{
	
	/**
	* Variable: method
	* The Flickr API method associated with this request.
	**/
	var method:String;
	/**
	* Variable: parameters
	* The parameters that will be passed to this request.
	**/
	var parameters:Object;
	/**
	* Variable: additionalArguments
	* Extra information about the reuqest which isn't held in the <parameters> variable but which is
	* needed when the response is received.
	**/
	var additionalArguments:Object;
	/**
	* Variable: requiresSigning
	* Whether this API method requires signing. Used by methods such
	* as flickr.auth.getFrob which need to be signed but are called
	* before the user is auth'd. Once a user is auth'd then all methods
	* are automatically signed.
	**/
	var requiresSigning:Boolean;
	/**
	* Variable: completed
	* Whether this request has been completed or not.
	**/
	var completed:Boolean;
	/**
	* Variable: id
	* The unique ID associated with this request.
	**/
	var id:Number;
	/**
	* Variable: queueId
	* The unique identifier which identifies this requests position in the <FlashrRequestQueue>
	**/
	var queueId:Number;
	
	/**
	* Variable: querystring
	* The querystring part of the url that was used to make this request.
	**/
	function get querystring():String
	{
		if (__querystring == undefined) {
			__querystring = _prepareQuerystring();
		}
		return __querystring;
	}
	function set querystring(q:String):Void
	{
	}
	private var __querystring:String;
	
	/**
	* Variable: _requests 
	* A static variable used to keep hold of all <FlashrRequest> Objects generated.
	**/
	private static var _requests:Array = [];
	
	/**
	* Function: FlashrRequest
	* Private constructor - use <generateRequest> to create <FlashrRequest> Objects as this will generate
	* the relevant unique ID that you will need to manipulate this request at a later date.
	* 
	* Parameters:
	* method					-	The Flickr API Method this request should call.
	* parameters				-	Any parameters that need to be passed to this method.
	* additionalArguments		-	Any additional arguments which aren't sent to flickr.com but which are
	* 								necessary to know what to associate the response with.
	* requiresSigning	 		-	Whether the called method requires signing. Used by methods such
	* 								as flickr.auth.getFrob which need to be signed but are called
	* 								before the user is auth'd. Once a user is auth'd then all methods
	* 								are automatically signed.
	**/
	private function FlashrRequest(method:String, parameters:Object, additionalArguments:Object, requiresSigning:Boolean)
	{
		this.method = method;
		this.parameters = parameters ? parameters : {};
		this.additionalArguments = additionalArguments ? additionalArguments : {};
		this.requiresSigning = requiresSigning;
		completed = false;
	}
	
	/**
	* Function: clone
	* Creates a copy of this <FlashrRequest>. Useful if you want to take the last request and change just
	* one parameter (e.g. the page number to skip through pages of results)
	* 
	* Returns:
	* The new <FlashrRequest> object.
	**/
	function clone():FlashrRequest
	{
		// better to copy the parameters and additionalArguments so that changes to one <FlashrRequest> won't effect the other.
		// TODO: Do additionalArguments need to be deeper copied?
		var newParameters:Object = {};
		for (var key:String in parameters) {
			newParameters[key] = parameters[key];
		}
		var newAdditionalArguments:Object = {};
		for (var key:String in additionalArguments) {
			newAdditionalArguments[key] = additionalArguments[key];
		}
		return generateRequest(method, newParameters, newAdditionalArguments, requiresSigning);
	}
	
	/**
	* Function: _prepareQuerystring
	* Private, internal function used to convert a list of arguments into a querystring for use in a URL 
	* for a call to flickr.com. Deals with signing the call as necessary. Can also be used when a 
	* signed querystring is needed outside of the API (e.g. for uploading).
	*
	* Returns:
	* The generated querystring.
	**/
	private function _prepareQuerystring()
	{
		parameters = parameters == undefined ? {} : parameters;
		if (parameters["api_sig"] != undefined) {
			delete parameters["api_sig"];
		}
		var _flashr:Flashr = Flashr.getFlashr();
		parameters["api_key"] = _flashr.apiKey;
		if (method) {
			parameters["method"] = method;
		}
		var querystring:String = "?";
		if (requiresSigning) { // this is a method that requires signing
			parameters["api_sig"] = _flashr._authSignCall(parameters);
		} else if (_flashr.authPerms != "none") { // the user is auth'd - token and sign the call
			parameters["auth_token"] = _flashr.authToken;
			parameters["api_sig"] = _flashr._authSignCall(parameters);
		}
		for (var argumentName:String in parameters) {
			querystring += argumentName+"="+parameters[argumentName] + "&";
		}
		return querystring;
	}
	
	
	/**
	* Function: generateRequest
	* Static function which generates a new <FlashrRequest> object and returns it's unique id.
	* 
	* Parameters:
	* method					-	The Flickr API Method this request should call.
	* parameters				-	Any parameters that need to be passed to this method.
	* additionalArguments		-	Any additional arguments which aren't sent to flickr.com but which are
	* 								necessary to know what to associate the response with.
	* requiresSigning	 		-	Whether the called method requires signing. Used by methods such
	* 								as flickr.auth.getFrob which need to be signed but are called
	* 								before the user is auth'd. Once a user is auth'd then all methods
	* 								are automatically signed.
	* 								
	* Returns:
	* The generated <FlashrRequest>
	**/
	static function generateRequest(method:String, parameters:Object, additionalArguments:Object, requiresSigning:Boolean):FlashrRequest
	{
		var fr:FlashrRequest = new FlashrRequest(method, parameters, additionalArguments, requiresSigning);
		_requests.push(fr);
		fr.id = _requests.length;
		LogWrapper.getLog().debug(fr+" generated");
		return fr;
	}
	
	/**
	* Function: getRequest
	* Static function which will return a reference to a <FlashrRequest> Object given it's unique <id>.
	* 
	* Parameters:
	* id						-	The <id> of the <FlashrRequest> Object you want.
	* 
	* Returns:
	* The relevant <FlashrRequest> Object or undefined if it was an illegal id.
	**/
	static function getRequest(id:Number):FlashrRequest
	{
		return _requests[id];
	}
	
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flashr.core.FlashrRequest - "+id+" {"+method+"}]";
	}
}