﻿import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.flashr.core.Photo;

/**
* Class: com.kelvinluck.flashr.core.Group
* Class to describe a group from Flickr.
* 
* Instances of this class are created to hold groups who have been
* returned in data from calls to the Flickr API.
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck < kelvin at kelvinluck dot com >
**/
class com.kelvinluck.flashr.core.Group {
	
	/**
	* Variable: PRIVACY_PRIVATE
	* Constant for <privacy> value for private groups.
	**/
	static var PRIVACY_PRIVATE:Number = 1;
	/**
	* Variable: PRIVACY_INVITE_ONLY
	* Constant for <privacy> value for invite only groups.
	**/
	static var PRIVACY_INVITE_ONLY:Number = 2;
	/**
	* Variable: PRIVACY_PUBLIC
	* Constant for <privacy> value for public groups.
	**/
	static var PRIVACY_PUBLIC:Number = 3;
	
	/**
	* Variable: nsid
	* This group's nsid.
	**/
	public var nsid:String;
	/**
	* Variable: name
	* The name of this group.
	**/
	public var name:String;
	/**
	* Variable: eighteenPlus
	* Whether this group is visible to members over 18 only.
	**/
	public var eighteenPlus:Boolean;
	/**
	* Variable: description
	* The description of this <Group>.
	**/
	var description:String;
	/**
	* Variable: members
	* The number of members in this <Group>.
	**/
	var members:Number;
	/**
	* Variable: privacy
	* The privacy setting of this group.
	* 
	* See Also:
	* <PRIVACY_PRIVATE>
	* 
	* <PRIVACY_INVITE_ONLY>
	* 
	* <PRIVACY_PUBLIC>
	**/
	var privacy:Number;
	/**
	* Variable: _pool
	* This <Group>s photo pool.
	**/
	private var _pool:Array;
	/**
	* Variable: numPhotos
	* The number of <Photo>s in this <Group>s pool.
	**/
	var numPhotos:Number;
	/**
	* Variable: _groups
	* A private static Object containing Group objects. Used by <getGroup> to insure that only one Group
	* is created for each nsid returned from flickr.com
	**/
	private static var _groups:Object = {};
	
	/**
	* Function: Group
	* Constructor function - creates a new Group object. 
	* 
	* Don't call directly, use <getGroup> to make sure that there is only one Group instance
	* for each nsid as returned by flickr.com
	* 
	* Parameters:
	* nsid				-	The nsid of the <Group> you are creating.
	**/
	private function Group(nsid:String)
	{
		this.nsid = nsid;
		_pool = [];
		LogWrapper.getLog().debug(this+" created");
	}
	
	/**
	* Function: addPhoto
	* Adds a <Photo> to this <Group>s <_pool>.
	* 
	* Parameters:
	* photo				-	The <Photo> to add.
	**/
	function addPhoto(photo:Photo)
	{
		if (photo == undefined) { // to deal with when we add photos based on <Flashr.groupsPoolsGetContext> and they are at the beginning or end of a group.
			return;
		}
		for (var i:Number=0; i<_pool.length; i++) {
			if (_pool[i].id == photo.id) {
				return;
			}
		}
		if (numPhotos) {
			numPhotos++;
		} else {
			numPhotos = _pool.length;
		}
		_pool.push(photo);
		photo.addGroup(this);
	}
	/**
	* Function: removePhoto
	* Removes a <Photo> from this <Group>s <_pool>.
	* 
	* Parameters:
	* photo				-	The <Photo> to remove.
	**/
	function removePhoto(photo:Photo)
	{
		for (var i:Number=0; i<_pool.length; i++) {
			if (_pool[i].id == photo.id) {
				_pool.splice(i, 1);
				return;
			}
		}
	}
	/**
	* Function: getPhotos
	* Get's this <Group>s <_pool>
	* 
	* Returns:
	* This <Group>s <_pool>
	**/
	function getPhotos():Array
	{
		return _pool;
	}
	
	/**
	* Function: getGroup
	* Get's a Group object for the given nsid.
	* 
	* Consults <_groups> to make sure that only one Group instance is created for each
	* nsid from flickr.com
	* 
	* Parameters:
	* nsid				-	The nsid of the Group you want to get.
	**/
	public static function getGroup(nsid:String):Group
	{
		if (_groups[nsid] == undefined) {
			_groups[nsid] = new Group(nsid);
		}
		return _groups[nsid];
	}
	
	function toString():String
	{
		return "[Object com.kelvinluck.flashr.core.Group - "+nsid+"]";
	}
}