﻿import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.util.DateFormat;

import com.kelvinluck.flashr.core.Flashr;
import com.kelvinluck.flashr.core.Person;
import com.kelvinluck.flashr.core.Note;
import com.kelvinluck.flashr.core.Tag;
import com.kelvinluck.flashr.core.PhotoSize;
import com.kelvinluck.flashr.core.Photoset;
import com.kelvinluck.flashr.core.Group;
import com.kelvinluck.flashr.core.Comment;

/**
* Class: com.kelvinluck.flashr.core.Photo
* Class to describe a photo from Flickr.
* 
* Instances of this class are created to hold photos who have been
* returned in data from calls to the Flickr API.
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck < kelvin at kelvinluck dot com >
**/
class com.kelvinluck.flashr.core.Photo {
	
	/**
	* Variable: id
	* This photo's id.
	**/
	public var id:String;
	/**
	* Variable: owner
	* A reference to the <Person> who owns this photo.
	**/
	public var owner:Person;
	/**
	* Variable: secret
	* This photo's secret.
	**/
	public var secret:String;
	/**
	* Variable: server
	* This photo's server.
	**/
	public var server:Number;
	/**
	* Variable: originalFormat
	* The format that this <Photo> was originally uploaded in.
	**/
	public var originalFormat:String;
	/**
	* Variable: title
	* This photo's title.
	**/
	public var title:String;
	/**
	* Variable: description
	* This photo's description.
	**/
	public var description:String;
	/**
	* Variable: numComments
	* The number of comments this photo has.
	**/
	public var numComments:Number;
	/**
	* Variable: _comments
	* The <Comment>s that have been left on this photo. Private so access through the
	* functions below.
	* 
	* See Also:
	* <getComments>
	* 
	* <addComment>
	**/
	private var _comments:Array;
	/**
	* Variable: _notes
	* The <Note>s that have been left on this photo. Private so access through the
	* functions below.
	* 
	* See Also:
	* <getNotes>
	* 
	* <addNote>
	**/
	private var _notes:Object;
	/**
	* Variable: _tags
	* The <Tag>s that are associated with this photo. Private so access through the
	* functions below.
	* 
	* See Also:
	* <getTags>
	* 
	* <addTag>
	**/
	var _tags:Object;
	/**
	* Variable: _sizes
	* The <PhotoSize>s that are associated with this photo. 
	* 
	* See Also:
	* <addSize>
	**/
	private var _sizes:Array;
	/**
	* Variable: isPublic
	* Whether or not this photo is public.
	**/
	public var isPublic:Boolean;
	
	// TODO - the following stuff isn't really relevant to a photo as such....
	// it should be stored as the relationship between the calling user and the
	// photo
	
	/**
	* Variable: isFriend
	* Whether or not this photo is by a friend.
	**/
	public var isFriend:Boolean;
	/**
	* Variable: isFamily
	* Whether or not this photo is by a family member.
	**/
	public var isFamily:Boolean;
	
	// /TODO comment about being about the relationship between the photo
	// and calling user rather than just the photo...
	
	/**
	* Variable: dateUploaded
	* The date this photo was uploaded
	* 
	* You set this as a String which the setter method automagically transforms from a
	* UNIX Timestamp (as provided by flickr.com) to a Date object.
	**/
	private var _dateUploaded:Date;
	public function set dateUploaded(date_str) 
	{
		_dateUploaded = new Date(date_str*1000);
	}
	public function get dateUploaded():Date
	{
		return _dateUploaded;
	}
	/**
	* Variable: dateTaken
	* The date this photo was taken.
	* 
	* You set this as a string which the setter method automagically transforms from a
	* MySQL Timestamp (as provided by flickr.com) to a Date object.
	**/
	private var _dateTaken:Date;
	public function set dateTaken(date_str) {
		_dateTaken = DateFormat.isoToDate(date_str);
	}
	public function get dateTaken():Date
	{
		return _dateTaken;
	}
	/**
	* Variable: dateUpdated
	* The date this photo was uploaded
	* 
	* You set this as a String which the setter method automagically transforms from a
	* UNIX Timestamp (as provided by flickr.com) to a Date object.
	**/
	private var _dateUpdated:Date;
	public function set dateUpdated(date_str) 
	{
		_dateUpdated = new Date(date_str*1000);
	}
	public function get dateUpdated():Date
	{
		return _dateUpdated;
	}
	/**
	* Variable: licence
	**/
	public var licence:Number;
	/**
	* Variable: rotation
	**/
	public var rotation:Number;
	/**
	* Variable: photoPageUrl
	* The link to this photo's page on flickr.com. If flickr.photos.getInfo has
	* been called on this photo then the link will be the "pretty" one with the
	* user's chosen screen name. Otherwise it will be the "ugly" one which works
	* just as well but doesn't look as good.
	**/
	private var _photoPageUrl:String;
	public function set photoPageUrl(photoPageUrl:String)
	{
		_photoPageUrl = photoPageUrl;
	}
	public function get photoPageUrl():String
	{
		if (_photoPageUrl) {
			return _photoPageUrl;
		}
		return "http://www.flickr.com/photos/"+owner.nsid+"/"+id+"/";
	}
	public function get _urlStart():String
	{
		return "http://static.flickr.com/"+server+"/"+id+"_"+secret;
	}
	/**
	* Variable: thumbnailUrl
	* The URL for the thumbnail image representing this <Photo>. *NOTE* - This is the 75 x 75 pixel
	* thumbnail image refered to as "small square" in the flickr.com documentation not the 100 pixels on
	* the longest side image called "thumbnail" in the flickr.com documentation. Why? For consistancy after an
	* inital mistake on my part!
	**/
	private var _thumbnailUrl:String;
	public function set thumbnailUrl(thumbnailUrl:String) {
		// would not normally want to do this but photosetsGetContext returns it and we don't necessarily know
		// the images server at that point so can't generate the URL as we normally would...
		_thumbnailUrl = thumbnailUrl;
	}
	public function get thumbnailUrl():String
	{
		if (_thumbnailUrl) {
			return _thumbnailUrl;
		} else {
			return _urlStart+"_s.jpg";
		}
	}
	/**
	* Variable: smallUrl
	* The URL for the small sized image (240px on longest side)
	**/
	public function set smallUrl(smallUrl:String) {}
	public function get smallUrl():String
	{
		return _urlStart+"_m.jpg";
	}
	/**
	* Variable: mediumUrl
	* The URL for the medium sized image (500px on longest side)
	**/
	public function set mediumUrl(mediumUrl:String) {}
	public function get mediumUrl():String
	{
		return _urlStart+".jpg";
	}
	/**
	* Variable: largeUrl
	* The URL for the large sized image (1024px on longest side)
	* 
	* Note:
	* This image only exists where the originally uploaded image is of sufficient size - your code should
	* be able to deal with the image not existing
	**/
	public function set largeUrl(largeUrl:String) {	}
	public function get largeUrl():String
	{
		return _urlStart+"_b.jpg";
	}
	/**
	* Variable: originalUrl
	* The URL for the originally uploaded image - returns an empty string if we don't 
	* know the format of the original image.
	* 
	* Note:
	* If this method returns an empty string you should do one of the following:
	* * Make an API call returning information about this photo passing "original_format" to the extras parameter.
	* * Call <Flashr.photosGetSizes>.
	* * Assume that the original image was a jpeg and use: (code)photo._urlStart+"_o.jpg";(end) to try and load the image.
	**/
	private var _originalUrl:String;
	public function set originalUrl(originalUrl:String) {
		// so if we have called photos.getSizes and haven't got the original_format from anywhere
		// we can still get the URL of the original image...
		_originalUrl = originalUrl;
	}
	public function get originalUrl():String
	{
		if (originalFormat) {
			return _urlStart+"_o."+originalFormat;
		}
		if (_originalUrl) {
			return _originalUrl;
		}
		LogWrapper.getLog().warn("Calling Photo.originalUrl when we don't know the url of the original photo.");
		return "";
	}
	
	
	// TODO - this is contextual again and so should not be stored with the Photo
	// but somewhere else refering to the photo object?
	/**
	* Variable: contextUrl
	* The URL for this photo within the context of the photostream e.g. as returned from
	* a call to flickr.photo.getContext
	**/
	public var contextUrl:String;
	/**
	* Variable: contextThumbUrl
	* The URL for this photo's thumbnail image within some sort of context e.g. as returned from
	* a call to flickr.photo.getContext
	* TODO: The thumbnail URL should be independant of context, no?
	**/
	public var contextThumbUrl:String;
	/**
	* Variable: contextPhotoPrevious
	* The previous photo in this context (as returned by <Flashr.photosGetContext>)
	**/
	public var contextPhotoPrevious:Photo;
	/**
	* Variable: contextPhotoNext
	* The next photo in this context (as returned by <Flashr.photosGetContext>)
	**/
	public var contextPhotoNext:Photo;
	/**
	* Variable: contextCount
	* Not documented on flickr.com but maybe the number of photos in this context (e.g. the user's photostream)?
	* (as returned by <Flashr.photoGetContext>)
	**/
	public var contextCount:Number;
	/**
	* Variable: contextPhotosets
	* Stores information about this <Photo>s context in given <Photoset>s
	* 
	* See Also:
	* <Flashr.photosetsGetContext>
	* 
	* <getContextPhotoset>
	* 
	* <setContextPhotoset>
	**/
	var contextPhotosets:Object;
	/**
	* Variable: contextGroupPool
	* Stores information about this <Photo>s context in given <Group>s
	* 
	* See Also:
	* <Flashr.groupsPoolsGetContext>
	* 
	* <getContextGroupPool>
	* 
	* <setContextGroupPool>
	**/
	var contextGroupPools:Object;
	
	// /TODO contextual comment above
	
	/**
	* Variable: _photosets
	* An Array containing references to the <Photoset>s that this <Photo> is in. Filled in by a call
	* to <Flashr.photosGetAllContexts> and also when you e.g. add a <Photo> to a <Photoset> through 
	* <Photoset.addPhoto> or <Flashr.photosetsEditPhotos>
	* 
	* See Also:
	* <addPhotoset>
	* 
	* <removePhotoset>
	* 
	* <getPhotosets>
	**/
	private var _photosets:Array;
	/**
	* Variable: _groups
	* An Array containing references to the <Group>s that this <Photo> is in. Filled in by a call to
	* <Flashr.photosGetAllContexts> and also when you e.g. add a <Photo> to a <Group> through
	* <Group.addPhoto>
	* 
	* See Also:
	* <addGroup>
	* 
	* <removeGroup>
	* 
	* <getGroups>
	**/
	var _groups:Array;
	
	/**
	* Variable: exifData
	* The EXIF/TIFF/GPS tags associated with this photo.
	* It is an Array of <ExifData> objects.
	**/
	public var exifData:Array;
	/**
	* Variable: _photos
	* A private static Object containing Photo objects. Used by <getPhoto> to insure that only 
	* one Photo is created for each photo ID returned from flickr.com
	* 
	* See Also:
	* <Flashr.photosGetExif>
	**/
	private static var _photos:Object = {};
	
	public var getInfoCalled:Boolean = false;
	
	/**
	* Function: Photo
	* Constructor function - creates a new Photo object.
	* 
	* Should be called via <Photo.getPhoto>
	**/
	private function Photo(id:String)
	{
		this.id = id;
		_comments = [];
		_notes = {};
		_tags = {};
		_sizes = [];
		contextPhotosets = {};
		_photosets = [];
		contextGroupPools = {};
		_groups = [];
		LogWrapper.getLog().debug(this+" created");
	}
	
	/**
	* Function: getComments
	* 
	* Returns an Array of all the <Comment>s left on this <Photo>. Will return an empty
	* Array until <Flashr.photosCommentsGetList> has been called.
	* 
	* If <Flashr.photosGetInfo> has been called and <Flashr.photosCommentsGetList> hasn't then
	* we know the number of comments there should be and so can raise a warning to remind the user
	* that they should have called <Flashr.photosCommentsGetList>
	* 
	* Returns:
	* All the <Comment>s that have been left on this <Photo>
	* 
	* See Also:
	* <_comments>
	* 
	* <Flashr.photosCommentsGetList>
	**/
	function getComments():Array
	{
		if (_comments.length == 0) {
			if (numComments != undefined && numComments != _comments.length) {
				LogWrapper.getLog().warn("Calling Photo.getComments without first calling photosCommentsGetList for this photo!");
			}
		}
		return _comments;
	}
	
	/**
	* Function: addComment
	* Adds a <Comment> object to this <Photo>.
	* 
	* Parameters:
	* note			-	The <Note> you want to add to this photo
	**/
	function addComment(comment:Comment):Void
	{
		_comments.push(comment);
	}
	/**
	* Function: getNotes
	* 
	* Returns an object containing all the note's that have been left on this
	* Photo
	* 
	* Returns:
	* All the <Note>'s that have been left on this Photo
	* 
	* See Also:
	* <_notes>
	* 
	* <Flashr.photosGetInfo>
	**/
	function getNotes():Object
	{
		return _notes;
	}
	
	/**
	* Function: addNote
	* Adds a <Note> object to this Photo.
	* 
	* Parameters:
	* note			-	The <Note> you want to add to this photo
	**/
	function addNote(note:Note):Void
	{
		if (_notes[note.id] == undefined) {
			_notes[note.id] = note;
		}
	}
	/**
	* Function: getTags
	* 
	* Returns an object containing all the tags's that are associated with this
	* Photo.
	* 
	* Returns:
	* All the <Tag>'s that are associated with this Photo. Each Tag is associated with
	* a <Person> as well &ndash; the <Person> who left this <Tag> on this <Photo>. Each <Tag> is
	* also associated with an id &ndash; this is the number you would use to call <Flashr.photosRemoveTag>
	* if you wanted to remove that <Tag> from this <Photo>.
	* 
	* See Also:
	* <_tags>
	* 
	* <getTagsAsStrings>
	* 
	* <Flashr.photosGetInfo>
	**/
	function getTags():Object
	{
		return _tags;
	}
	
	/**
	* Function: getTagsAsStrings
	* 
	* A utility function to get all the <Tag>'s associated with this <Photo> as Strings.
	* Use this in cases where you don't care about the details of the <Tag> or it's relationship to
	* this <Photo> (e.g. you aren't planning to modify it or delete it) and you just want to know what 
	* it is...
	* 
	* Returns:
	* An Array containing the clean version of the text for each <Tag> left on this <Photo>. NOTE: this 
	* is only useful to you if you are only interested in the textual content of these <Tag>s (otherwise 
	* use <getTags>) and will only be filled after a call to <Flashr.photosGetInfo> for this <Photo>.
	* 
	* See Also:
	* <_tags>
	* 
	* <getTags>
	* 
	* <Flashr.photosGetInfo>
	**/
	function getTagsAsStrings():Array
	{
		var returnTags:Array = [];
		for(var id:String in _tags) {
			var t:Tag = _tags[id].tag;
			returnTags.push(t.value);
		}
		return returnTags;
	}
	
	/**
	* Function: addTag
	* Adds a <Tag> object to this Photo and associates it with a <Person> who is the <Person> who
	* associated that <Tag> with this <Photo>.
	* 
	* Parameters:
	* id			-	The id which describes the link between this <Photo> and <Tag>.
	* tag			-	The <Tag> you want to add to this photo.
	* author		-	The <Person> who attached that <Tag> to this <Photo>.
	* 
	* See Also:
	* <_tags>
	* 
	* <getTags>
	**/
	function addTag(id:String, tag:Tag, author:Person):Void
	{
		if (_tags[id] == undefined) {
			_tags[id] = {tag:tag, author:author, id:id};
		}
	}
	
	/**
	* Function: setTags
	* Set's this Photo's <_tags>...
	* 
	* Parameters:
	* tags			-	The tags to add to this photo...
	* author		-	The <Person> who added these tags (leave blank if you don't know)
	* 
	* TODO:
	* Doesn't deal properly with tags with spaces in the name now :(
	* 
	* Also need to get the <Person> object for the logged in user so we can correctly
	* associate the author with the tag...
	* 
	* We don't know the id's that will associate the <Tag>'s with the <Photo>s at this
	* point so are using garbage ones. This means that you won't be able to remove the
	* tags using those ids...
	**/
	function setTags(tags:String, author:Person)
	{
		_tags = {};
		var tagsSplit:Array = tags.split(" ");
		for (var i:Number=0; i<tagsSplit.length; i++) {
			var thisTag:Tag = Tag.getTag(tagsSplit[i]);
			// It would be good to call Tag.addPhoto at this stage but we don't know 
			// the id associating the Person with the Tag
			addTag(i+"", thisTag, author);
			Flashr.getFlashr().authUser.addTag(thisTag);
		}
	}
	/**
	* Function: addPhotoset
	* Adds a photoset to this <Photo>s <_photosets> Array
	* 
	* Parameters:
	* photoset				-	The <Photoset> to add to this <Photo>.
	**/
	function addPhotoset(photoset:Photoset)
	{
		for (var i:Number=0; i<_photosets.length; i++) {
			if (_photosets[i].id == photoset.id) {
				return;
			}
		}
		_photosets.push(photoset);
	}
	/**
	* Function: removePhotoset
	* Removes a given <Photoset> from this <Photo>s <_photosets> Array.
	**/
	function removePhotoset(photoset:Photoset)
	{
		for (var i:Number=0; i<_photosets.length; i++) {
			if (_photosets[i].id == photoset.id) {
				_photosets.splice(i, 1);
			}
		}
	}
	/**
	* Function: getPhotosets
	* 
	* Returns:
	* This <Photo>s <_photosets> Array.
	* 
	* See Also:
	* <_photosets>
	**/
	function getPhotosets():Array
	{
		return _photosets;
	}
	/**
	* Function: addGroup
	* Adds a <Group> to this <Photo>s <_groups> Array
	* 
	* Parameters:
	* group					-	The <Group> to add to this <Photo>.
	**/
	function addGroup(group:Group)
	{
		for (var i:Number=0; i<_groups.length; i++) {
			if (_groups[i].nsid == _groups.nsid) {
				return;
			}
		}
		_groups.push(group);
	}
	/**
	* Function: removeGroup
	* Removes a given <Group> from this <Photo>s <_groups> Array.
	**/
	function removeGroup(group:Group)
	{
		for (var i:Number=0; i<_groups.length; i++) {
			if (_groups[i].nsid == group.nsid) {
				_groups.splice(i, 1);
			}
		}
	}
	/**
	* Function: getGroups
	* 
	* Returns:
	* This <Group>s <_groups> Array.
	* 
	* See Also:
	* <_groups>
	**/
	function getGroups():Array
	{
		return _groups;
	}
	/**
	* Function: setContextPhotoset
	* Adds information about this <Photo>s context in a given <Photoset>
	* 
	* Parameters:
	* photosetId			-	The id of the <Photoset> this context is for
	* previousPhoto			-	The <Photo> before this <Photo> in this context or undefined if this
	* 							is the first <Photo> in this context.
	* nextPhoto				-	The <Photo> after this <Photo> in this context or undefined if this is
	* 							the last <Photo> in this context.
	**/
	function setContextPhotoset(photosetId:Number, previousPhoto:Photo, nextPhoto:Photo)
	{
		if (contextPhotosets[photosetId] == undefined) {
			contextPhotosets[photosetId] = {};
		}
		contextPhotosets[photosetId].previousPhoto = previousPhoto;
		contextPhotosets[photosetId].nextPhoto = nextPhoto;
	}
	/**
	* Function: getContextPhotoset
	* Get information about the context of this <Photo> in a given <Photoset>
	* 
	* Parameters:
	* photosetId			-	The id of the <Photoset> you want the context information for.
	* 
	* Returns:
	* An object containing nextPhoto and previousPhoto which are <Photo> objects. undefined if there is no
	* information about this <Photo>s context in the requested <Photoset>
	**/
	function getContextPhotoset(photosetId:Number)
	{
		return contextPhotosets[photosetId];
	}
	/**
	* Function: setContextGroupPool
	* Adds information about this <Photo>s context in a given <Group>
	* 
	* Parameters:
	* groupId				-	The id of the <Group> this context is for
	* previousPhoto			-	The <Photo> before this <Photo> in this context or undefined if this
	* 							is the first <Photo> in this context.
	* nextPhoto				-	The <Photo> after this <Photo> in this context or undefined if this is
	* 							the last <Photo> in this context.
	**/
	function setContextGroupPool(groupId:String, previousPhoto:Photo, nextPhoto:Photo)
	{
		if (contextGroupPools[groupId] == undefined) {
			contextGroupPools[groupId] = {};
		}
		contextGroupPools[groupId].previousPhoto = previousPhoto;
		contextGroupPools[groupId].nextPhoto = nextPhoto;
	}
	/**
	* Function: getContextGroupPool
	* Get information about the context of this <Photo> in a given <Photoset>
	* 
	* Parameters:
	* groupId				-	The id of the <Group> you want the context information for.
	* 
	* Returns:
	* An object containing nextPhoto and previousPhoto which are <Photo> objects. undefined if there is no
	* information about this <Photo>s context in the requested <Group>
	**/
	function getContextGroupPool(groupId:Number)
	{
		return contextGroupPools[groupId];
	}
	
	/**
	* Function: addSize
	* Adds a <PhotoSize> object to this Photo.
	* 
	* Parameters:
	* size			-	The <PhotoSize> you want to add to this photo.
	**/
	function addSize(size:PhotoSize):Void
	{
		_sizes.push(size);
	}
	/**
	* Function: getSizes
	* Get's information about the available <PhotoSize>s for this <Photo> (assuming 
	* <Flashr.photosGetSizes> has been called - otherwise returns an empty Array.
	* 
	* See Also:
	* <_sizes>
	* 
	* <Flashr.photosGetSizes>
	* 
	* Returns :
	* The <_sizes> Array.
	**/
	function getSizes():Array
	{
		return _sizes;
	}
	
	
	/**
	* Function: getPhoto
	* Get's a Photo object for the given photo ID.
	* 
	* Consults <_photos> to make sure that only one Photo instance is created for each
	* photo ID from flickr.com
	* 
	* Parameters:
	* photoId		-	The id of the Photo you want to get
	* 
	* Returns:
	* The <Photo> object for te given id
	**/
	public static function getPhoto(photoId:String):Photo
	{
		if (_photos[photoId] == undefined) {
			_photos[photoId] = new Photo(photoId);
		}
		return _photos[photoId];
	}
	
	function toString():String
	{
		return "[Object com.kelvinluck.flashr.core.Photo - "+id+"]";
	}
}