﻿import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.flashr.core.Person;
import com.kelvinluck.flashr.core.Photo;

/**
* Class: com.kelvinluck.flashr.core.Tag
* Class to describe a tag on a photo on flickr.com
* 
* Instances of this class are created to hold tags that are attached to Photos
* returned from flickr.com
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck < kelvin at kelvinluck dot com >
**/
class com.kelvinluck.flashr.core.Tag {
	
	/**
	* Variable: raw
	* The raw text of this tag
	**/
	public var raw:String;
	/**
	* Variable: value
	* The text of this tag
	**/
	private var _value:String;
	function get value():String
	{
		if (_value != undefined) {
			return _value;
		}
		return raw;
	}
	function set value(newValue:String):Void
	{
		_value = newValue;
	}
	/**
	* Variable: _photos
	* An object detailing the relationships between this tag and different photos.
	**/
	public var _photos:Object;
	/**
	* Variable: relatedTags
	* An object containing a list of tags that are related to this one, based on clustered usage analysis.
	* 
	* See Also:
	* <Flashr.tagsGetRelated>
	**/
	public var relatedTags:Object;
	
	/**
	* Variable: _tags
	* A private static Object containing Tag objects. Used by <getTag> to insure that only one Tag
	* is created for each tag returned from flickr.com
	**/
	private static var _tags:Object = {};
	
	/**
	* Function: Tag
	* Constructor function - creates a new Tag object.
	* 
	* Don't call directly, use <getTag> to make sure that there is only one Tag instance
	* for each id as returned by flickr.com
	**/
	private function Tag(raw:String)
	{
		this.raw = raw;
		this._photos = {};
		this.relatedTags = {};
		LogWrapper.getLog().debug(this+" created");
	}
	
	/**
	* Function: addPhoto
	* Add's a reference to a <Photo> to this <Tag>.
	* 
	* Parameters:
	* id			-	The id which describes the link between this <Photo> and <Tag>.
	* photo			-	The <Photo> you want to add to this tag.
	* author		-	The <Person> who attached that <Tag> to this <Photo>.
	* 
	* See Also:
	* <_photos>
	* 
	* <getPhotos>
	**/
	
	function addPhoto(id:String, photo:Photo, author:Person):Void
	{
		if (_photos[id] == undefined) {
			_photos[id] = {photo:photo, author:author, id:id};
		}
	}
	/**
	* Function: getPhotos
	* Get's this tags <_photos> Object.
	* 
	* See Also:
	* <_photos>
	* 
	* <setPhotos>
	**/
	function getPhotos():Object
	{
		return _photos;
	}
	
	/**
	* Function: addRelatedTag
	* Add's a reference to a related <Tag> to this Tag.
	* 
	* See Also:
	* <Flashr.tagsGetRelated>
	**/
	function addRelatedTag(tag:Tag)
	{
		// stored indexed by their raw element to ensure each Tag is only added once...
		relatedTags[tag.raw] = tag;
	}
	
	/**
	* Function: getTag
	* Get's a Tag object for the given id.
	* 
	* Consults <_tags> to make sure that only one Tag instance is created for each
	* tag id from flickr.com
	* 
	* Parameters:
	* raw			-	The raw of the Tag you want to get
	**/
	public static function getTag(raw:String):Tag
	{
		if (_tags[raw] == undefined) {
			_tags[raw] = new Tag(raw);
		}
		return _tags[raw];
	}
	
	function toString():String
	{
		return "[Object com.kelvinluck.flashr.core.Tag - "+raw+"]";
	}
}