/**
* Class: com.kelvinluck.util.SOSLogPublisher
* Class to add the ability to output logged data to the PowerFlasher
* Socket Output Server.
* 
* Mostly just a simple conversion of Pablo Costantini's 
* LuminicBox.Log.TracePublisher code.
* 
* Requirements:
* This code requires that the LuminicBox.Log classes are in your classpath
* when you compile it and that you have a SOS Server running to see the
* output on. Also bear in mind that connections to the SOS Server are subject
* to Flash's security restrictions.
* 
* Usage:
* (start code)
* import com.kelvinluck.util.LogWrapper;
* import com.kelvinluck.util.SOSLogPublisher;
* LogWrapper.getInstance().init();
* LogWrapper.getLog().addPublisher(new SOSLogPublisher("myAppsName"));
* LogWrapper.getLog().fatal("This is a fatal error :'(");
* LogWrapper.getLog().error("This is an error :(");
* LogWrapper.getLog().warn("This is a warning, warning, warning");
* LogWrapper.getLog().info("This is information");
* LogWrapper.getLog().debug("This is debugging info");
* LogWrapper.getLog().debug([1,2,3,{a:"Part A", b:"Part B"}, "Some text", 99]);
* (end)
* 
* See Also:
* <http://sos.powerflasher.com/>
* 
* <http://sos.powerflasher.de/doc/1-04/>
* 
* <http://www.luminicbox.com/blog/?page=post&id=2>
* 
* <http://kelvinluck.com/article/logging-from-fame>
* 
* Author:
* Kelvin Luck derived from the work of Pablo Costantini
* 
* License:
* Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*      http://www.mozilla.org/MPL/MPL-1.1.html
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
import LuminicBox.Log.IPublisher;
import LuminicBox.Log.LogEvent;
import LuminicBox.Utils.StringUtility;

class com.kelvinluck.util.SOSLogPublisher implements IPublisher {
	
	/**
	* Variable: COL_FATAL
	* The colour of the background of log messages with a severity of FATAL
	**/
	private static var COL_FATAL:Number = 0xFF0000;
	/**
	* Variable: COL_ERROR
	* The colour of the background of log messages with a severity of ERROR
	**/
	private static var COL_ERROR:Number = 0xFF6666;
	/**
	* Variable: COL_WARN
	* The colour of the background of log messages with a severity of WARN
	**/
	private static var COL_WARN:Number = 0xFFCCCC;
	/**
	* Variable: COL_INFO
	* The colour of the background of log messages with a severity of INFO
	**/
	private static var COL_INFO:Number = 0xDDDDDD;
	/**
	* Variable: COL_DEBUG
	* The colour of the background of log messages with a severity of DEBUG
	**/
	private static var COL_DEBUG:Number = 0xFFFFFF;
	
	/**
	* Variable: indent
	* Sets the string that is used to indent object blocks. Defaults to
	* a tab but you can use anything you like ("|   " is good if you are
	* logging to a monospace font in SOS)
	*/
	var indent:String = "\t";
	//var indent:String = "|   ";
	
	private static var TERMINATOR:String = "\n";
	private static var _allSockets:Array;
	
	private var _maxDepth:Number;
	private var _sosServer:String;
	private var _sosPort:Number;
	private var _socket:XMLSocket;
	
	/**
	* Variable: maxDepth
	* The maximum inspection depth.
	* 
	* The default value is 4.
	* The max. valid value is 255.
	*/
	function set maxDepth(value:Number) { 
		_maxDepth = (_maxDepth>255)?255:value; 
	}
	function get maxDepth():Number { 
		return _maxDepth; 
	}
	
	/**
	* Function: SOSLogPublisher
	* Creates a SOSLogPublisher instance. Also initalises communications with SOS
	* through an XMLSocket.
	* 
	* Parameters:
	* appName				-	The name of this app.
	* 							[defaults to "UNKNOWN"]
	* sosServer				-	The IP address of the SOS server.
	* 							[defaults to localhost]
	* sosPort				-	The command port of the SOS server to connect to.
	* 							[defaults to 4444 - the default Log-Port];
	* maxDepth				-	The maximum depth to inspect into objects / arrays.
	* 							[defaults to 4]
	*/
	function SOSLogPublisher(appName:String, sosServer:String, sosPort:Number, maxDepth:Number) {
		_sosServer = (sosServer ? sosServer : "127.0.0.1");
		_sosPort = (sosPort ? sosPort : 4444);
		_maxDepth = (maxDepth ? maxDepth : 4);
		_socket = new XMLSocket();
		_socket.connect(_sosServer, _sosPort);
		appName = appName ? appName : "UNKNOWN";
		_sendCommand("<appName>"+appName+"</appName>");
		_sendCommand("<setKey><name>FATAL</name><color>"+COL_FATAL+"</color></setKey>");
		_sendCommand("<setKey><name>ERROR</name><color>"+COL_ERROR+"</color></setKey>");
		_sendCommand("<setKey><name>WARN</name><color>"+COL_WARN+"</color></setKey>");
		_sendCommand("<setKey><name>INFO</name><color>"+COL_INFO+"</color></setKey>");
		_sendCommand("<setKey><name>DEBUG</name><color>"+COL_DEBUG+"</color></setKey>");
		if (_allSockets == undefined) {
			_allSockets = [];
		}
		_allSockets.push(_socket);
	}
	
	/**
	* Function: _sendCommand
	* Internal, private function that makes it easier to send a command without worrying 
	* about the null byte or other implementation details.
	* 
	* Also makes it easier to switch between sending the command on the Command-Port 
	* (which seems to make sense but doesn't activate the userful "Bring to 
	* front > During connection" functionality of SOS) and sending commands on the
	* Log-Port (which has the overhead of 4 bytes per command).
	* 
	* Arguments:
	* command					-	The XML to send as the command.
	*/
	private function _sendCommand(command:String)
	{
		// comment out the following line if you set sosPort to be the Command-Port...
		command = "!SOS"+command;
		_socket.send(command + TERMINATOR);
	}
	
	/**
	* Function: publish
	* Sends a log event through the socket to the SOS Server
	* 
	* Parameters:
	* e						-	The <LogEvent> to log...
	*/
	function publish(e:LogEvent):Void {
		var arg:Object = e.argument;
		//var txt:String = "*" + e.level.toString().charAt(0) + "*";
		var txt:String = "";
		if(e.loggerId) txt += e.loggerId;
		txt += ":";
		txt += analyzeObj(arg,1);
		var txtSplit:Array = txt.split("\n");
		if (txtSplit.length == 1) {
			_sendCommand("<showMessage key=\""+e.level.toString()+"\"><![CDATA["+txt+"]]></showMessage>");
		} else {
			var firstLine:String = txtSplit[0];
			_sendCommand("<showFoldMessage key=\""+e.level.toString()+"\"><title><![CDATA["+firstLine+"]]></title><message><![CDATA["+txtSplit.splice(1).join("\n")+"]]></message></showFoldMessage>");
		}
	}

	/**
	* Function: analyzeObj
	* Takes an anything and returns a pretty printed version by recursively moving down the
	* object tree
	* 
	* Taken from the original LuminicBox.Log.TracePublisher code and slightly modified...
	* 
	* Parameters:
	* o						-	The object to analyse
	* depth					-	How deep into the object tree we currently are
	* 
	* See Also:
	* <LuminicBox.Log.TracePublisher>
	*/
	private function analyzeObj(o,depth:Number):String {
		var txt:String = "";
		var typeOf:String = typeof(o);
		if(typeOf == "string") {
			// STRING
			txt += "\"" + o + "\"";
		} else if(typeOf == "boolean" || typeOf == "number") {
			// BOOLEAN / NUMBER
			txt += o;
		} else if(typeOf == "undefined" || typeOf == "null") {
			// UNDEFINED / NULL
			txt += "("+typeOf+")";
		} else {
			// OBJECT
			var stringifyObj:Boolean = false;
			var analize:Boolean = true;
			if(o instanceof Array) {
				// ARRAY
				typeOf = "array";
				stringifyObj = false;
			} else if(o instanceof Button) {
				// BUTTON
				typeOf = "button";
				stringifyObj = true;
			} else if(o instanceof Date) {
				// DATE
				typeOf = "date";
				analize = false;
				stringifyObj = true;
			} else if(o instanceof Color) {
				// COLOR
				typeOf = "color";
				analize = false;
				stringifyObj = true;
				o = o.getRGB().toString(16);
			} else if(o instanceof MovieClip) {
				// MOVIECLIP
				typeOf = "movieclip";
				stringifyObj = true;
			} else if(o instanceof XML) {
				// XML
				typeOf = "xml";
				analize = false;
				stringifyObj = true;
			} else if(o instanceof XMLNode) {
				// XML
				typeOf = "xmlnode";
				analize = false;
				stringifyObj = true;
			}
			txt += "(";
			if (typeOf == "object") txt += o; // o.toString is implicitly called.
			else if (typeOf == "array") txt += "array[" + o.length + "]";
			else txt += typeOf;
			txt += ") ";
			if(stringifyObj) txt += o.toString();
			if(analize && depth <= _maxDepth) {
				var txtProps = "";
				for(var prop in o) {
					txtProps += "\n" +
						StringUtility.multiply( indent, (depth+1) ) +
						prop + ":" +
						analyzeObj(o[prop], (depth+1) );
				}
				if(txtProps.length > 0) txt += "{" + txtProps + "\n" + StringUtility.multiply( indent, depth ) + "}";
			}
		}
		return txt;
	}
	
	/**
	* Function: closeAllSockets
	* Loops over all created <SOSLogPublisher>s and shuts down their
	* connections. Use if e.g. you want to kill the connection to the
	* SOS Server (so that it stops sitting on top) without killing the 
	* swf you are testing...
	*/
	public static function closeAllSockets():Void
	{
		for (var i:Number = 0; i<_allSockets.length; i++) {
			_allSockets[i].close();
		}
	}
	
	/**
	* Function: toString
	* 
	* Returns:
	* The name of the class of this object.
	*/
	function toString():String { 
		return "[Object com.kelvinluck.util.SOSLogPublisher]"; 
	}	
}